#ifndef __XAUDIOXAPOTEST_
#define __XAUDIOXAPOTEST_

#include <math.h>

#include <xaudio2.h>
#include "XAPOBase.h"

void rdft( int n, int isgn, double *a, int *ip, double *w );

#define BUFFERSIZE ( 1024 * 16 )
#define FFTSIZE ( 4096 )

double Power[FFTSIZE / 2 + 1];
double UnitFreq = 0;

class __declspec( uuid("{5EB8D611-FF96-429d-8365-2DDF89A7C1CD}")) 
XAudioXAPOTest : public CXAPOBase{
public:
	XAudioXAPOTest( const XAPO_REGISTRATION_PROPERTIES* pRegistrationProperties ) : CXAPOBase( pRegistrationProperties ){};

	STDMETHOD( LockForProcess )( UINT32 InputLockedParameterCount, __in_ecount_opt(InputLockedParameterCount) const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pInputLockedParameters, UINT32 OutputLockedParameterCount, __in_ecount_opt(OutputLockedParameterCount) const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pOutputLockedParameters )
	{
		memcpy( &WaveformatEx, pInputLockedParameters[0].pFormat, sizeof( WAVEFORMATEX ) );
		UnitFreq = ( (double)WaveformatEx.nSamplesPerSec / 2 ) / ( FFTSIZE / 2 + 1 );

		pBuffer = new float [BUFFERSIZE]; // K
		memset( pBuffer, 0, BUFFERSIZE * sizeof( float ) );
		pBufferTmp = new float [BUFFERSIZE]; // K
		pFFTBuffer = new double [FFTSIZE];
		pFFTWorkArea = new int [2 + (int)sqrt( (double)( FFTSIZE / 2 ) ) + 10]; // 10̓}[W
		pFFTSinTable = new double [FFTSIZE / 2];
		IsFFTInitialized = false;

		return S_OK;
	};

	STDMETHOD_( void, UnlockForProcess )()
	{
		IsFFTInitialized = false;

		delete [] pFFTSinTable;
		delete [] pFFTWorkArea;
		delete [] pFFTBuffer;
		delete [] pBufferTmp;
		delete [] pBuffer;
	};

	STDMETHOD_( void, Process )( THIS_ UINT32 InputProcessParameterCount, __in_ecount_opt(InputProcessParameterCount) const XAPO_PROCESS_BUFFER_PARAMETERS* pInputProcessParameters, UINT32 OutputProcessParameterCount, __inout_ecount_opt(OutputProcessParameterCount) XAPO_PROCESS_BUFFER_PARAMETERS* pOutputProcessParameters, BOOL IsEnabled )
	{
		memcpy( pBufferTmp, pBuffer, BUFFERSIZE * sizeof( float ) );

		int DataLength = pInputProcessParameters[0].ValidFrameCount * WaveformatEx.nChannels;

		memcpy( pBuffer, pBufferTmp + DataLength, ( BUFFERSIZE - DataLength ) * sizeof( float ) );
		memcpy( pBuffer + BUFFERSIZE - DataLength, pInputProcessParameters[0].pBuffer, DataLength * sizeof( float ) );
//printf( "%f\n", ( (float*)pInputProcessParameters[0].pBuffer )[0] );

		for( int i = 0; i < FFTSIZE; ++i ) pFFTBuffer[i] = pBuffer[i * WaveformatEx.nChannels + 0];
//		for( int i = 0; i < FFTSIZE; ++i ) pFFTBuffer[i] = ( pBuffer[i * WaveformatEx.nChannels + 0] + pBuffer[i * WaveformatEx.nChannels + 1] ) / 2;

		if( !IsFFTInitialized ){
			pFFTWorkArea[0] = 0;
			rdft( FFTSIZE, 1, pFFTBuffer, pFFTWorkArea, pFFTSinTable );

			IsFFTInitialized = true;
		}
		else{
			rdft( FFTSIZE, 1, pFFTBuffer, pFFTWorkArea, pFFTSinTable );
		}

		Power[0] = pow( pFFTBuffer[0], 2 );
		for( int i = 1; i < FFTSIZE / 2; ++i ){
			Power[i] = pow( pFFTBuffer[i * 2 + 0], 2 ) + pow( pFFTBuffer[i * 2 + 1], 2 );
		}
		Power[FFTSIZE / 2] = pow( pFFTBuffer[1], 2 );

//printf( "%f\n", Power[0] );

//		if( pInputProcessParameters->BufferFlags == XAPO_BUFFER_VALID ){
//			for( UINT32 i = 0; i < pInputProcessParameters[0].ValidFrameCount * WaveformatEx.nChannels; ++i ) ( (float*)pInputProcessParameters[0].pBuffer )[i] *= 0.1f;
//		}
	};

private:
	WAVEFORMATEX WaveformatEx;
	float *pBuffer, *pBufferTmp;
	double *pFFTBuffer, *pFFTSinTable;
	int *pFFTWorkArea;

	bool IsFFTInitialized;
};

#endif
